from typing import Any

from django.conf import settings
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.http import HttpRequest

from django_features.custom_fields.models import CustomFieldBaseModel
from django_features.custom_fields.models import CustomFieldTypeBaseModel


class BaseAdmin(admin.ModelAdmin):
    def has_module_permission(self, request: HttpRequest) -> bool:
        return settings.CUSTOM_FIELD_ADMIN and settings.CUSTOM_FIELDS_FEATURE


class CustomFieldBaseAdmin(admin.ModelAdmin):
    def get_form(self, request: HttpRequest, obj: Any = None, **kwargs: Any) -> Any:
        form = super().get_form(request, obj, **kwargs)
        form.base_fields["content_type"].queryset = ContentType.objects.filter(
            id__in=[
                content_type.id
                for content_type in ContentType.objects.all()
                if content_type.model_class() is not None
                and issubclass(content_type.model_class(), CustomFieldBaseModel)
            ]
        )
        form.base_fields["type_content_type"].queryset = ContentType.objects.filter(
            id__in=[
                content_type.id
                for content_type in ContentType.objects.all()
                if content_type.model_class() is not None
                and issubclass(content_type.model_class(), CustomFieldTypeBaseModel)
            ]
        )
        return form

    def get_readonly_fields(self, request: HttpRequest, obj: Any = None) -> list[str]:
        if obj:
            return [*self.readonly_fields, "field_type"]
        return self.readonly_fields



__all__ = [
    "get_custom_field_model",
    "get_custom_value_model",
    "clear_custom_field_model_cache",
]


from functools import lru_cache

from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from django_features.custom_fields.models.field import AbstractBaseCustomField
from django_features.custom_fields.models.value import AbstractBaseCustomValue


@lru_cache(maxsize=1)
def get_custom_field_model() -> type[AbstractBaseCustomField]:
    """
    Return the CustomField model that is active in this project.
    """
    if settings.CUSTOM_FIELD_MODEL is None:
        raise ImproperlyConfigured(
            "CUSTOM_FIELD_MODEL and CUSTOM_FIELD_VALUE_MODEL must be defined in settings to use the custom fields app."
        )

    try:
        return django_apps.get_model(settings.CUSTOM_FIELD_MODEL, require_ready=False)  # type: ignore[unused-ignore]
    except ValueError:
        raise ImproperlyConfigured(
            "CUSTOM_FIELD_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "CUSTOM_FIELD_MODEL refers to model '%s' that has not been installed"
            % settings.CUSTOM_FIELD_MODEL
        )


@lru_cache(maxsize=1)
def get_custom_value_model() -> type[AbstractBaseCustomValue]:
    """
    Return the CustomValue model that is active in this project.
    """
    if settings.CUSTOM_FIELD_VALUE_MODEL is None:
        raise ImproperlyConfigured(
            "CUSTOM_FIELD_MODEL and CUSTOM_FIELD_VALUE_MODEL must be defined in settings to use the custom fields app."
        )

    try:
        return django_apps.get_model(  # type: ignore[unused-ignore]
            settings.CUSTOM_FIELD_VALUE_MODEL, require_ready=False
        )
    except ValueError:
        raise ImproperlyConfigured(
            "CUSTOM_FIELD_VALUE_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "CUSTOM_FIELD_VALUE_MODEL refers to model '%s' that has not been installed"
            % settings.CUSTOM_FIELD_VALUE_MODEL
        )


def clear_custom_field_model_cache() -> None:
    """
    Clear cached model lookups for custom fields.
    """
    get_custom_field_model.cache_clear()
    get_custom_value_model.cache_clear()
